class SoAColumnRows:
  
  EPOCH_ROW = 0
  CYCLE_ROW = 1
  CYCLE_START_ROW = 2
  CYCLE_PERIOD_ROW = 3
  CYCLE_END_RULE_ROW = 4
  TIMING_ROW = 5
  VISIT_LABEL_ROW = 6
  VISIT_WINDOW_ROW = 7

  HEADER_ROW = 8
  FIRST_ACTIVITY_ROW = 9

  ACTIVITY_COL = 0
  CHILD_ACTIVITY_COL = 1
  BC_COL = 2
  FIRST_VISIT_COL = 3
